import { z } from 'zod';

export const revisionTagSchema = z.object({
	type: z.enum(['alpha', 'beta', 'rc', 'stable']),
	version: z.string()
});

export const fileInlineSchema = z.object({
	type: z.literal('file'),
	inline: z.literal(true),
	metafile: z.boolean(),
	content: z.string().max(256 * 1024),
	sha1: z.string().regex(/^[a-f0-9]{40}$/),
	sha512: z.string().regex(/^[a-f0-9]{128}$/)
});

export const fileBlobSchema = z.object({
	type: z.literal('file'),
	inline: z.literal(false),
	sha1: z.string().regex(/^[a-f0-9]{40}$/),
	sha512: z.string().regex(/^[a-f0-9]{128}$/)
});

export const fileSchema = z.union([fileInlineSchema, fileBlobSchema]);

export const fileTreeSchema = z.lazy(() => z.union([folderSchema, fileSchema]));

export const folderSchema: z.ZodType<{
	type: 'folder';
	children: Record<string, z.infer<typeof fileTreeSchema>>;
}> = z.object({
	type: z.literal('folder'),
	children: z.record(z.string().regex(/^[^/]+$/), fileTreeSchema)
});

export const diffSetSchema = z.object({
	path: z
		.string()
		.regex(/^[^/]+$/)
		.array(),
	type: z.literal('set'),
	value: fileTreeSchema
});

export const diffDeleteSchema = z.object({
	path: z
		.string()
		.regex(/^[^/]+$/)
		.array(),
	type: z.literal('delete')
});

export const diffSchema = z.union([diffSetSchema, diffDeleteSchema]);

export const revisionDiffSchema = z.object({
	id: z.string().regex(/^[A-Za-z0-9_-]{21}$/),
	tag: revisionTagSchema.optional(),
	type: z.literal('diff'),
	tree: diffSchema.array(),
	message: z.string()
});

export const revisionBaseSchema = z.object({
	id: z.string().regex(/^[A-Za-z0-9_-]{21}$/),
	tag: revisionTagSchema.optional(),
	type: z.literal('base'),
	tree: fileTreeSchema,
	message: z.string()
});

export const revisionSchema = z.union([revisionBaseSchema, revisionDiffSchema]);

export const modpackSchema = z.object({
	_id: z.string().regex(/^[A-Za-z0-9_-]{21}$/),
	type: z.literal('modpack'),
	name: z.string(),
	author: z.string().email(),
	revisions: revisionSchema.array()
});
